local cv_showitemtimers = CV_RegisterVar({
    name = "em_showitemtimer",
    defaultvalue = "Yes",
    possiblevalue = CV_YesNo,
})

local cache = {}

local function cachePatches(v, name, patches)
    cache[name] = {}

    for i = 1, #patches do
        table.insert(cache[name], v.cachePatch(patches[i]))
    end

    return cache[name]
end

hud.add(function(v,p,c)
	if HugeQuest then return end
	if splitscreen then return end
	if leveltime < 2 then return end

	if not p.spectator and cv_showitemtimers.value then

		-- name - for cache
        -- timer - timer to use
        -- patches - list of patch names
        -- anim_frames - frames for 1 animation step (used only for invincibility, in fact)
		local timerTable = {
			{
                name = "superform",
                timer = p.kartstuff[k_invincibilitytimer],
				hqsuperflag = p.hugequest.super,
                patches = {"K_ISEMR1", "K_ISEMR2", "K_ISEMR3", "K_ISEMR4"},
                anim_frames = 3
            }
		}
		-- sort table by timer

		table.sort(timerTable, function(a, b)
			if(a.timer ~= nil and b.timer ~= nil) then 
				return a.timer > b.timer 
			end 
		end)

		local hardY = --[[(hpmod and hpmod.running) and 140 or ]] 170
		local iconX = (hpmod and hpmod.running) and 240 or 150

		local iconXOffset = -30
		for i, icon in ipairs(timerTable) do
			if(icon.timer and icon.timer > 0) and icon.hqsuperflag then
				iconXOffset = $ + 30
			end
		end

		iconX = $ - (iconXOffset/2)
		local hasBeenOffset = false
		for i, icon in ipairs(timerTable) do

			-- Draw icon/relevant timer

			local timer

			if icon.timer and icon.timer > 0 and icon.hqsuperflag then

				timer = icon.timer
				
				local minutes = G_TicsToMinutes(timer, true)
                local seconds = G_TicsToSeconds(timer)
                local centiseconds = G_TicsToCentiseconds(timer)
				
				local timerstring = (minutes*60)+seconds.."."..(centiseconds < 9 and "0" or "")..centiseconds
				local transflags = V_SNAPTOBOTTOM
				if(timer < 3) then
					transflags = (leveltime % 5) and $|TR_TRANS30 or $
				end

                local patches = cache[icon.name]

                if patches == nil then
                    patches = cachePatches(v, icon.name, icon.patches)
                end

                local patch_num = (leveltime % (icon.anim_frames * #patches) / icon.anim_frames) + 1
				
				if icon.color then
					v.drawScaled(iconX*FRACUNIT, hardY*FRACUNIT, 6*FRACUNIT/10, patches[patch_num], transflags, v.getColormap(-1, icon.color))
				else
					v.drawScaled(iconX*FRACUNIT, hardY*FRACUNIT, 6*FRACUNIT/10, patches[patch_num], transflags)
				end
				v.drawString(iconX + 7, hardY + 15, timerstring, transflags, "thin")

				iconX = $ + 30
			end


		end

	end
end)
